@@ -447,7 +447,7 @@ gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
         #tr.StS<-sum(S^2)
         tr.StS<- sum(diag(S%*%diag(wt2)%*%t(S)%*% diag(1/wt2)))
         ###edf is different from the definition in Chris' code
-        #edf<-dp.n-2*tr.S+tr.StS
+        edf<-dp.n-2*tr.S+tr.StS
         yhat<-gw_fitted(x, betas)
         residual<-y-exp(yhat)
         ########rss <- sum((y - gwr.fitted(x,b))^2)
@@ -469,7 +469,7 @@ gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
         #gwR2.adj<-1-(1-gw.R2)*(dp.n-1)/(edf-1) #Adjusted R squared valu
         
         pseudo.R2 <- 1- gw.dev/null.dev
-        GW.diagnostic<-list(gw.deviance=gw.dev,AICc=AICc,AIC=AIC,pseudo.R2 =pseudo.R2)        
+        GW.diagnostic<-list(gw.deviance=gw.dev,AICc=AICc,AIC=AIC,pseudo.R2 =pseudo.R2,edf=edf)        
      }
      else
      {
@@ -674,7 +674,7 @@ gwr.binomial <- function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-
 ##Author: BL	
 print.ggwrm<-function(x, ...)
 {
-  if(class(x) != "ggwrm") stop("It's not a gwm object")
+  if(!inherits(x, "ggwrm")) stop("It's not a gwm object")
   cat("   ***********************************************************************\n")
   cat("   *                       Package   GWmodel                             *\n")
   cat("   ***********************************************************************\n")
